@@ -21,6 +21,7 @@
 #include <net/ipv6.h>
 #include <net/addrconf.h>
 #include <net/ipv6_stubs.h>
+#include <net/ndisc.h>
 
 /* alternative VLAN for IP session 0 if not untagged */
 #define MBIM_IPS0_VID	4094
@@ -102,13 +103,8 @@ static const struct net_device_ops cdc_mbim_netdev_ops = {
 #else
 	.ndo_tx_timeout = bp_usbnet_tx_timeout,
 #endif
-
-#if LINUX_VERSION_IS_GEQ(4,11,0)
-	.ndo_get_stats64      = dev_get_tstats64,
-#else
-	.ndo_get_stats64 = bp_dev_get_tstats64,
-#endif
 	
+	.ndo_get_stats64      = dev_get_tstats64,
 	.ndo_change_mtu       = cdc_ncm_change_mtu,
 	.ndo_set_mac_address  = eth_mac_addr,
 	.ndo_validate_addr    = eth_validate_addr,
